/*
     File:       DICOM_Toolkit_Parser.h

     Contains:   Escape DICOM Toolkit Parser Component Interfaces.

     Copyright:  (c)2002-2004 by Escape Information Services, all rights reserved.

     Bugs?       For support or bug reports please send email to support@escape.gr
 
*/
#ifndef __DICOM_TOOLKIT_PARSER__
#define __DICOM_TOOLKIT_PARSER__

#ifndef __MACH__
	#ifndef __CONDITIONALMACROS__
		#include <ConditionalMacros.h>
	#endif

	#ifndef __MACTYPES__
		#include <MacTypes.h>
	#endif

	#ifndef __COMPONENTS__
		#include <Components.h>
	#endif
#endif

#ifndef __DICOM_TOOLKIT_TYPES__
	#include "DICOM_Toolkit_Types.h"
#endif

#ifdef __cplusplus
    extern "C" {
#endif

#if PRAGMA_IMPORT
#pragma import on
#endif

/*!
    @header DICOM_Toolkit_Parser.h
    @discussion This header includes the declarations of the functions of the DICOM Parser component.
*/

#pragma export on

/*!
    @function DTkParseValidateDataRef
    @abstract Validates a data reference of given type
    @discussion
    This function is provided for applications to determine whether a particular file contains DICOM formatted data. 
    The thoroughness of the validation is controlled by the validationFlags parameter.
    
    DICOM files are not yet associated by the DICOM standard with a particular extension, so the actual 
    data must be examined to make this determination. 
    
    @param ci              [I] The parser component instance
    @param dataRef         [I] Data reference to validate
    @param dataRefType     [I] Type of data reference
    @param validationFlags [I] Flags which control the thoroughness of the validation.
    @param valid           [O] Pointer to boolean whose value indicates whether a data reference is valid (true) or not (false)
    @result Error code
*/
EXTERN_API(ComponentResult) DTkParseValidateDataRef(
                                                     DTkParserComponentInstance ci, 
                                                     Handle  dataRef, 
                                                     OSType  dataRefType, 
                                                     UInt32 validationFlags,  
                                                     Boolean *valid);



/*!
    @function DTkParseGetDictInfoForTag
    @abstract Retrieves DICOM Dictionary information concerning a data element tag
    @param ci                 [I] The parser component instance
    @param elementTagPtr      [I] Pointer to a DICOM element's tag structure
    @param elementTagDictInfo [O] Address of the DTkTagDictInfo structure to be filled by the function.
    @result Error code
*/
EXTERN_API(ComponentResult) DTkParseGetDictInfoForTag(
                                                       DTkParserComponentInstance ci,
                                                       DTkElementTagPtr elementTagPtr,
                                                       DTkTagDictInfoPtr elementTagDictInfo);



/*!
    @function DTkParseSetDataReference
    @abstract Sets the data reference (e.g. file or URL) via which the Parser component will access the data in the DICOM file
    @param ci          [I] The parser component instance
    @param dataRef     [I] Data reference to be set
    @param dataRefType [I] Type of the data reference to be set
    @result Error code
*/
EXTERN_API(ComponentResult) DTkParseSetDataReference(
                                                      DTkParserComponentInstance ci, 
                                                      Handle dataRef, 
                                                      OSType dataRefType);



/*!
    @function DTkParseGetDataReference
    @abstract Gets the data reference via which the Parser component accessess the data in the DICOM file
    @param ci          [I] The parser component instance
    @param dataRef     [O] Data reference
    @param dataRefType [O] Type of the data reference
    @result Error code
*/
EXTERN_API(ComponentResult) DTkParseGetDataReference(
                                                      DTkParserComponentInstance ci, 
                                                      Handle *dataRef, 
                                                      OSType *dataRefType);



/*!
    @function DTkParseSetParserFlags
    @abstract Sets flags controlling various aspects of the parsing process
    @discussion <UL>
                <LI>This function should be called at the beginning of a parsing session, immediately after validation.
                <LI>Please see the documentation of individual flags for their meaning.
                <LI>The flags recognized are those defined in the DTkParserFlags enumeration.
                </UL>
    @param ci          [I] The parser component instance
    @param parserFlags [I] The flags to be set
    @result Error code
*/
EXTERN_API(ComponentResult) DTkParseSetParserFlags(
                                                    DTkParserComponentInstance ci, 
                                                    UInt32  parserFlags);



/*!
    @function DTkParseGetParserFlags
    @abstract Retrieves the flags set by DTkParseSetParserFlags
    @param ci          [I] The parser component instance
    @param parserFlags [O] Pointer to the retrieved flags
    @result Error code
*/
EXTERN_API(ComponentResult) DTkParseGetParserFlags(
                                                    DTkParserComponentInstance ci, 
                                                    UInt32 *parserFlags);


/*!
    @function DTkParseGetFileMetaInformation
    @abstract Retrieves meta information concerning the data in the DICOM file
    @discussion Valid values for the selector are those specified in the DTkMetaInformationSelector enumeration.
                The correspondence between selector and type of the return value is given in the following table:
                <DIV  align = "CENTER">
                <TABLE border = "1" width="90%">
                    <thead> <TR><TH>Selector                        </TH><TH>Type                   </TH></TR></thead>
                            <TR><TD>kDTkEndiannessSelector          </TD><TD>DTkEndianness          </TD></TR>
                            <TR><TD>kDTkPixelRepresentationSelector </TD><TD>DTkPixelRepresentation </TD></TR>
                            <TR><TD>kDTkDataVRVerbositySelector     </TD><TD>DTkDataVRVerbosity     </TD></TR>
                            <TR><TD>kDTkDataTransferSyntaxSelector  </TD><TD>DTkTransferSyntax      </TD></TR>
                            <TR><TD>kDTkBasicOffsetTableTypeSelector</TD><TD>DTkBasicOffsetTableType</TD></TR>
                            <TR><TD>kDTkPixelDataLengthTypeSelector </TD><TD>DTkPixelDataLengthType </TD></TR>
               </TABLE>
                </DIV>
    @param ci       [I] The parser component instance
    @param selector [I] Selector which specifies kind of information to be retrieved.
    @param value    [O] Pointer to a value appropriate to the selector
    @result Error code
*/
EXTERN_API(ComponentResult) DTkParseGetFileMetaInformation(
                                                            DTkParserComponentInstance ci,
                                                            DTkMetaInformationSelector selector,
                                                            void *value);



/*!
    @function DTkParseFindElement
    @abstract Retrieves the reference number of an element given its tag and parent element reference number
    @param ci               [I] The parser component instance
    @param parent           [I] The reference number of the parent element
    @param elementTagRecPtr [I] Pointer to element's tag structure
    @param elementRefPtr    [O] Pointer to the found element's reference number
    @result Error code
*/
EXTERN_API(ComponentResult) DTkParseFindElement(
                                                 DTkParserComponentInstance ci,
                                                 DTkElementRef parent,
                                                 DTkElementTagPtr elementTagRecPtr,
                                                 DTkElementRefPtr elementRefPtr);



/*!
    @function DTkParseGetElementRefs
    @abstract Retrieves the reference numbers of all elements of a certain parent element 
              given the parent element's reference number
    @discussion The reference number of the parent element can either be zero, a reference number 
                of a sequence or a reference number of a sequence item. 
                When it is zero the reference numbers of the top-level elements are returned.
                The caller is responsible for disposing the returned DTkElemRefArrayHandle.
    @param ci              [I] The parser component instance
    @param parent          [I] The reference number of the parent element
    @param elementRefArray [O] Pointer to the retrieved array of element reference numbers
    @result Error code
*/
EXTERN_API(ComponentResult) DTkParseGetElementRefs(
                                                    DTkParserComponentInstance ci, 
                                                    DTkElementRef parent, 
                                                    DTkElemRefArrayHandlePtr elementRefArray);



/*!
    @function DTkParseGetElementTag
    @abstract Retrieves the tag of a DICOM data element referenced by elemRef
    @param ci            [I] The parser component instance
    @param elemRef       [I] The reference number of the data element whose tag is to be retrieved
    @param elementTagPtr [O] Address of the DTkElementTag structure to be filled by the function
    @result Error code
*/
EXTERN_API(ComponentResult) DTkParseGetElementTag(
                                                   DTkParserComponentInstance ci, 
                                                   DTkElementRef elemRef, 
                                                   DTkElementTagPtr elementTagPtr);



/*!
    @function DTkParseGetElementDataInfo 
    @abstract Retrieves information concerning a data element referenced by elemRef by parsing the DICOM file
    @param ci              [I] The parser component instance
    @param elemRef         [I] The reference number of the data element
    @param elementDataInfo [O] Address of the DTkElementDataInfo struct to be filled by the function
    @result Error code
*/
EXTERN_API(ComponentResult) DTkParseGetElementDataInfo(
                                                        DTkParserComponentInstance ci,
                                                        DTkElementRef elemRef,
                                                        DTkElementDataInfoPtr elementDataInfo);



/*!
    @function DTkParseGetElementData
    @abstract Retrieves the raw contnent of a data element referenced by elemRef
    @param ci             [I] The parser component instance
    @param elemRef        [I] The reference number of the data element
    @param dataSize       [I] The size of the elementDataPtr pointer
    @param elementDataPtr [O] The pointer to be filled by the function. The pointer belongs to the caller.
    @result Error code
*/
EXTERN_API(ComponentResult) DTkParseGetElementData(
                                                    DTkParserComponentInstance ci, 
                                                    DTkElementRef elemRef, 
                                                    UInt32 dataSize, 
                                                    void *elementDataPtr);



/*!
    @function DTkParseGetElementDataAsText
    @abstract Retrieves the value of a data element's item of index itemIndex as a string
    @discussion It is probably better to use the more general DTkParseGetElementDataAsValue, which does not do to any data coercion.
    @param ci        [I] The parser component instance
    @param elemRef   [I] The reference number of the data element
    @param itemIndex [I] The one-based index of an item whose value is to be retrieved
    @param valueL    [O] A handle allocated by the function which contains the retrieved text. It is the caller's responsibility to dispose of this handle.
    @result Error code
*/
EXTERN_API(ComponentResult) DTkParseGetElementDataAsText(
                                                          DTkParserComponentInstance ci,
                                                          DTkElementRef elemRef, 
                                                          SInt32 itemIndex, 
                                                          Handle *textHandlePtr);



/*!
    @function DTkParseGetElementDataAsValue
    @abstract Retrieves the value of a numeric data element's item of index itemIndex as a value along with its type 
    @discussion The values returned are endian swapped, if necessary, so as to correspond to the architecture on which the executable is running.
                When the value returned is of Handle type, it is the responsibility of the caller to dispose of it.
    @param ci          [I] The parser component instance
    @param elemRef     [I] The reference number of the data element
    @param itemIndex   [I] The one-based index of an item whose value is to be retrieved
    @param valueRecPtr [O] Address of DTkNumericValueRecord structure to be filled by the function
    @result Error code
*/
EXTERN_API(ComponentResult)     DTkParseGetElementDataAsValue(
                                                               DTkParserComponentInstance ci, 
                                                               DTkElementRef elemRef, 
                                                               SInt32 itemIndex, 
                                                               DTkNumericValueRecordPtr valueRecPtr);



/*!
    @function DTkParseGetDIRElementRefs
    @abstract Retrieves the reference numbers of all the directory record sequence items that are subordinate to the parent element.
    @discussion The reference number of the parent element can either be zero or a directory record sequence item. 
                When it is zero the element references returned are those of the PATIENT directory records.
    @param ci              [I] The parser component instance
    @param parent          [I] The reference number of the parent element
    @param elementRefArray [O] Pointer to the retrieved array of element reference numbers (directory record sequence items)
    @result Error code
*/
EXTERN_API(ComponentResult) DTkParseGetDIRElementRefs(
                                                       DTkParserComponentInstance ci, 
                                                       DTkElementRef parent, 
                                                       DTkElemRefArrayHandlePtr elementRefArray);


/*!
    @function DTkParseGetNumberOfFrames
    @abstract Retrieves the number of frames of the DICOM file corresponding to the parser component instance.
    @discussion When the flags parameter is zero the call returns the actual number of frames in the file. When the 
                kGetNumberOfFramesFlag_GetNominalNumber flag is set the nominal number of frames in the file is
                returned. The two retrieved values may differ with DICOM files that are incomplete, e.g. because of
                a file transfer error.
    @param ci        [I] The parser component instance
    @param flags     [I] Flags specializing the behavior of the call. Applicable flags are defined in the DTkParseGetNumberOfFramesFlags enumeration.
    @param numFrames [O] Pointer to the retrieved number of frames
    @result Error code
*/
EXTERN_API(ComponentResult) DTkParseGetNumberOfFrames(
													   DTkParserComponentInstance ci, 
													   UInt32 flags,
													   UInt32 *numFrames);

#pragma export off

#ifdef PRAGMA_IMPORT_OFF
#pragma import off
#elif PRAGMA_IMPORT
#pragma import reset
#endif

#ifdef __cplusplus
    }
#endif

#endif // __DICOM_TOOLKIT_PARSER__
