/*
     File:       DICOM_Toolkit_VOIBlitter.h

     Contains:   Escape DICOM Toolkit VOIBlitter Component Interfaces.

     Copyright:  (c)2002-2004 by Escape Information Services, all rights reserved.

     Bugs?       For support or bug reports please send email to support@escape.gr
 
*/
#ifndef __DICOM_TOOLKIT_VOIBLITTER__
#define __DICOM_TOOLKIT_VOIBLITTER__

#ifndef __MACH__
	#ifndef __CONDITIONALMACROS__
		#include <ConditionalMacros.h>
	#endif

	#ifndef __MACTYPES__
		#include <MacTypes.h>
	#endif

	#ifndef __COMPONENTS__
		#include <Components.h>
	#endif
#endif

#ifndef __DICOM_TOOLKIT_TYPES__
	#include "DICOM_Toolkit_Types.h"
#endif

#ifdef __cplusplus
    extern "C" {
#endif


#if PRAGMA_ENUM_ALWAYSINT
    #if defined(__fourbyteints__) && !__fourbyteints__ 
        #define __DICOM_TOOLKIT_VOIBLITTER__RESTORE_TWOBYTEINTS
        #pragma fourbyteints on
    #endif
    #pragma enumsalwaysint on
#elif PRAGMA_ENUM_OPTIONS
    #pragma option enum=int
#elif PRAGMA_ENUM_PACK
    #if __option(pack_enums)
        #define __DICOM_TOOLKIT_VOIBLITTER__RESTORE_PACKED_ENUMS
        #pragma options(!pack_enums)
    #endif
#endif

#if PRAGMA_IMPORT
#pragma import on
#endif

/*!
    @header DICOM_Toolkit_VOIBlitter.h
    @discussion This header includes the declarations of the functions of the DICOM VOIBlitter component.
*/

#pragma export on

typedef ComponentInstance DTkVOIBlitterComponentInstance;

typedef enum
{
    kVOIBlitter_DontOptimizeVOIBounds,
    kVOIBlitter_ConstrainVOIBoundsToUserDefinedPixelBounds,
    kVOIBlitter_UnkownVOIBoundsClippingOption				 = kDTkUnknown
} VOIBlitter_BoundsClippingOption;


/*!
	@function DTkVOIBlitSetVOIBoundsClippingOption
	@abstract Sets the pixel value clipping method to be used during VOI transformations.
   	@discussion When a VOI bliting function is invoked the values of the destination pixels are calculated according 
                to the following pseudo-code (for simplicity of presentation rounding effects are not included):

				<code>
                VOIWindowStart = VOIwindowCenter - VOIwindowWidth/2	
                VOIWindowEnd   = VOIWindowStart + VOIwindowWidth - 1

                if sourcePixel <= VOIWindowStart
                    destintationPixel = 0
                else if sourcePixel >= VOIWindowEnd
                    destintationPixel = 255
                else
                	destintationPixel = (sourcePixel - VOIWindowStart) * 255 / (VOIWindowStart - VOIWindowEnd)
				</code>

         		When VOIWindowStart in less that the minimum Pixel Value or VOIWindowEnd is greater than the maximum pixel value,
         		the actual pixel range will be mapped to a sub-range of the available output range, resulting in a compressed image.
         		
                To avoid this effect the VOI copy functions can optionally apply the following optimization to the VOI bounds:

				<code>
                if VOIWindowStart < minPixelValue
				{
					VOIWindowStart = minPixelValue
				}

				if VOIWindowEnd > maxPixelValue
				{
					*VOIWindowEnd = maxPixelValue
				}
				</code>
                The optimization results in mapping the minimum pixel value to 0 and the maximum pixel value to 255, 
                thus maximizing the resulting image's contrast.

				The kDTkDontOptimizeVOIBounds flag parameter instructs the function not to apply this optimization

				The kVOIBlitter_ConstrainVOIBoundsToUserDefinedPixelBounds flag parameter instructs the function to apply the 
				optimization using the pixel value bounds that have been supplied by the user of the API using the DTkVOIBlitSetVOIBounds call.

    @param ci             [I] The VOIBlitter component instance
    @param clippingOption [I] The clipping option. Allowed values are defined in the DTkVOIBoundsClippingOption enumeration.
    @result Error code
*/
EXTERN_API(ComponentResult) DTkVOIBlitSetVOIBoundsClippingOption(
                                                                 ComponentInstance ci, 
                                                                 VOIBlitter_BoundsClippingOption clippingOption);


/*!
	@function DTkVOIBlitGetVOIBoundsClippingOption
	@abstract Retrieves the currently active pixel value clipping method used during VOI transformations.
    @param ci             [I] The VOIBlitter component instance
    @param clippingOption [O] The retrieved clipping option. Allowed values are defined in the DTkVOIBoundsClippingOption enumeration.
    @result Error code
*/
EXTERN_API(ComponentResult) DTkVOIBlitGetVOIBoundsClippingOption(
                                                                 ComponentInstance ci, 
                                                                 VOIBlitter_BoundsClippingOption *clippingOption);


/*!
	@function DTkVOIBlitSetPixelBounds
	@abstract Sets the Pixel bounds to be used for clipping the VOI range produced by applying a specific window center and window width.
    @param ci            [I] The VOIBlitter component instance
    @param minPixelValue [I] The minimum pixel value
    @param maxPixelValue [I] The maximum pixel value
    @result Error code
*/
EXTERN_API(ComponentResult) DTkVOIBlitSetPixelBounds(
                                                   ComponentInstance ci, 
                                                   UInt32 minPixelValue, 
                                                   UInt32 maxPixelValue);

/*!
	@function DTkVOIBlitGetPixelBounds
	@abstract Retrieves the currently active Pixel bounds.
    @param ci            [I] The VOIBlitter component instance
    @param minPixelValue [O] The minimum pixel value
    @param maxPixelValue [O] The maximum pixel value
    @result Error code.
*/
EXTERN_API(ComponentResult) DTkVOIBlitGetPixelBounds(
                                                   ComponentInstance ci, 
                                                   UInt32 *minPixelValue, 
                                                   UInt32 *maxPixelValue);

/*!
    @function DTkVOIBlitVOICopyPixels
    @abstract Blits a source image buffer to a destination image buffer applying the given VOI parameters
    @discussion DTkVOIBlitVOICopyPixels is the function that applies the VOI transformation for DICOM
                images. Currently the source and destination rectangles must be of equal size.
    @param ci                  [I] The decompressor component instance
    @param displayWindowWidth  [I] The value of the Window Width to be applied
    @param displayWindowCenter [I] The value of the Window Level to be applied
    @param srcPixMapHandle     [I] A handle to the source pixel map
    @param dstPixMapHandle     [I] A handle to the destination pixel map
    @param srcRect             [I] The address of the source rectangle
    @param dstRect             [I] The address of the destination rectangle
    @result Error code
*/
EXTERN_API(ComponentResult) DTkVOIBlitVOICopyPixels(
                                                           ComponentInstance ci,
                                                           SInt32 displayWindowCenter,
                                                           UInt32 displayWindowWidth,
                                                           PixMapHandle srcPixMapHandle, 
                                                           PixMapHandle dstPixMapHandle, 
                                                           Rect *srcRect, 
                                                           Rect *dstRect );

typedef enum
{
	kDTkVOIBlitConvertUserToDisplay,
	kDTkVOIBlitConvertDisplayToUser,
	kDTkVOIBlitUnknownConversionDirection = kDTkUnknown
} DTkVOIBlitPixelValueConversionDirection;


/*!
	@function DTkVOIBlitConvertBetweenUserAndDisplayValues
	@abstract Converts a pixel value from the user to the display reference system
    @param ci 				        [I] The VOIBlitter component instance
	@param conversionDirection		[I] Specifies the direction for the conversion
	@param pixelFormat				[I] The pixel format of the data. Allowd values are k16GrayCodecType and k8IndexedGrayPixelFormat.
    @param photoInterpretation		[I] The photometric interpreation of the DICOM pixel data
	@param pixelValueReversalFactor [I] The value out of which pixel values have been subtracted during a pixel value reversal operation 
                                        when the photometric interpretation of the pixel data requires it
    @param pixelRangeShiftValue     [I] The value by which pixel values were shifted after the modality transformation in order
                                        to be brought into the positive range
    @param displayValue			  [I/O] The value of a pixel in the display value reference system
    @param userValue			  [I/O] The value of a pixel in the user value reference system    
    @result Error code.
*/
EXTERN_API(ComponentResult) DTkVOIBlitConvertBetweenUserAndDisplayValues(
                               ComponentInstance ci,                                                    
                               DTkVOIBlitPixelValueConversionDirection conversionDirection,
                               FourCharCode pixelFormat,
                               DTkPhotometricInterpretation photoInterpretation,
                               DTkPixelValueReversalFactorType pixelValueReversalFactor,
							   DTkPixelRangeShiftValueType pixelRangeShiftValue,
                               SInt32* displayValue,
                               Float32* userValue);


/*!
@function DTkVOIBlitGetVOIScale256
@abstract Returns a gray-scale color table corresponding to a gray-scale ramp to
          which a VOI transformation with the given parameters has been applied.
    @param ci 				   [I] The VOIBlitter component instance
    @param displayWindowCenter [I] The value of the Window Center to be applied
    @param displayWindowWidth  [I] The value of the Window Width to be applied
    @param DTkUInt8Vect_256Ptr [O] The output gray-scale color table. Input values are ignored. Caller supplies the buffer.
    @result Error code.
*/

EXTERN_API(ComponentResult) DTkVOIBlitGetVOIScale256(
                                ComponentInstance ci,                                                    
                                SInt32 displayWindowCenter,
                                UInt32 displayWindowWidth,
                                DTkUInt8Vect_256Ptr VOIScale256);

#pragma export off

#ifdef PRAGMA_IMPORT_OFF
#pragma import off
#elif PRAGMA_IMPORT
#pragma import reset
#endif

#if PRAGMA_ENUM_ALWAYSINT
    #pragma enumsalwaysint reset
    #ifdef __DICOM_TOOLKIT_VOIBLITTER__RESTORE_TWOBYTEINTS
        #pragma fourbyteints off
    #endif
#elif PRAGMA_ENUM_OPTIONS
    #pragma option enum=reset
#elif defined(__DICOM_TOOLKIT_VOIBLITTER__RESTORE_PACKED_ENUMS)
    #pragma options(pack_enums)
#endif

#ifdef __cplusplus
    }
#endif


#endif // __DICOM_TOOLKIT_VOIBLITTER__
